/*
-----------------------------------------------------------------------------
This source file is part of OGRE
(Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2005 The OGRE Team
Also see acknowledgements in Readme.html

You may use this sample code for anything you like, it is not covered by the
LGPL like the rest of the engine.
-----------------------------------------------------------------------------
*/

/*
-----------------------------------------------------------------------------
Filename:    tutorial2.h
Description: A place for me to try out stuff with OGRE.
-----------------------------------------------------------------------------
*/
#ifndef __tutorial2_h_
#define __tutorial2_h_

#include "ExampleApplication.h"


#include <OgreNoMemoryMacros.h>
#include <CEGUI.h>
#include <CEGUISystem.h>
#include <CEGUISchemeManager.h>
#include <OgreCEGUIRenderer.h>
#include <OgreMemoryMacros.h>

CEGUI::MouseButton convertOgreButtonToCegui(int buttonID)
{
	switch (buttonID)
	{
	case MouseEvent::BUTTON0_MASK:
		return CEGUI::LeftButton;
	case MouseEvent::BUTTON1_MASK:
		return CEGUI::RightButton;
	case MouseEvent::BUTTON2_MASK:
		return CEGUI::MiddleButton;
	case MouseEvent::BUTTON3_MASK:
		return CEGUI::X1Button;
	default:
		return CEGUI::LeftButton;
	}
}


class tutorial2FrameListener : public ExampleFrameListener, public MouseMotionListener, public MouseListener
{
private:
	SceneManager* mSceneMgr;
	CEGUI::Renderer* mGUIRenderer;
	bool mShutdownRequested;
public:
	tutorial2FrameListener(SceneManager *sceneMgr, RenderWindow* win, Camera* cam, CEGUI::Renderer* renderer)
		: ExampleFrameListener(win, cam, false, true),
		mGUIRenderer(renderer),
		mShutdownRequested(false),
		mSceneMgr(sceneMgr)
	{
		mEventProcessor->addMouseMotionListener(this);
		mEventProcessor->addMouseListener(this);
		mEventProcessor->addKeyListener(this);
	}

	bool frameStarted(const FrameEvent& evt)
	{
		bool ret = ExampleFrameListener::frameStarted(evt);
		if(mUseBufferedInputMouse)
		{
			CEGUI::MouseCursor::getSingleton().show( );
		}
		else
		{
			CEGUI::MouseCursor::getSingleton().hide( );
		}

		return ret;

	}

	void requestShutdown(void)
	{
		mShutdownRequested = true;
	}

	bool frameEnded(const FrameEvent& evt)
	{
		if (mShutdownRequested)
			return false;
		else
			return ExampleFrameListener::frameEnded(evt);
	}

	void mouseMoved (MouseEvent *e)
	{
		CEGUI::System::getSingleton().injectMouseMove(
			e->getRelX() * mGUIRenderer->getWidth(),
			e->getRelY() * mGUIRenderer->getHeight());
		e->consume();
	}

	void mouseDragged (MouseEvent *e)
	{
		mouseMoved(e);
	}

	void mousePressed (MouseEvent *e)
	{
		CEGUI::System::getSingleton().injectMouseButtonDown(
			convertOgreButtonToCegui(e->getButtonID()));
		e->consume();
	}

	void mouseReleased (MouseEvent *e)
	{
		CEGUI::System::getSingleton().injectMouseButtonUp(
			convertOgreButtonToCegui(e->getButtonID()));
		e->consume();
	}

	void mouseClicked(MouseEvent* e) {}
	void mouseEntered(MouseEvent* e) {}
	void mouseExited(MouseEvent* e) {}

	void keyPressed(KeyEvent* e)
	{
		if(e->getKey() == KC_ESCAPE)
		{
			mShutdownRequested = true;
			e->consume();
			return;
		}

		CEGUI::System::getSingleton().injectKeyDown(e->getKey());
		CEGUI::System::getSingleton().injectChar(e->getKeyChar());
		e->consume();
	}
};



class tutorial2App : public ExampleApplication
{
private:
	CEGUI::OgreCEGUIRenderer* mGUIRenderer;
	CEGUI::System* mGUISystem;
public:
	tutorial2App()
		: mGUIRenderer(0),
		mGUISystem(0)
	{}

	~tutorial2App()
	{
		if(mGUISystem)
		{
			delete mGUISystem;
			mGUISystem = 0;
		}
		if(mGUIRenderer)
		{
			delete mGUIRenderer;
			mGUIRenderer = 0;
		}
	}

protected:

	virtual void createCamera(void)
	{
		// Create the camera
		mCamera = mSceneMgr->createCamera("PlayerCam");

		// Position it at 500 in Z direction
		mCamera->setPosition(Vector3(0,0,80));
		// Look back along -Z
		mCamera->lookAt(Vector3(0,0,-300));
		mCamera->setNearClipDistance(5);
	}




	// Just override the mandatory create scene method
	virtual void createScene(void)
	{
		// setup GUI system
		mGUIRenderer = new CEGUI::OgreCEGUIRenderer(mWindow,
			Ogre::RENDER_QUEUE_OVERLAY, false, 3000, mSceneMgr);

		mGUISystem = new CEGUI::System(mGUIRenderer);

		CEGUI::Logger::getSingleton().setLoggingLevel(CEGUI::Informative);


		// load scheme and set up defaults
		CEGUI::SchemeManager::getSingleton().loadScheme(
			(CEGUI::utf8*)"TaharezLookSkin.scheme");
		mGUISystem->setDefaultMouseCursor(
			(CEGUI::utf8*)"TaharezLook", (CEGUI::utf8*)"MouseArrow");
		mGUISystem->setDefaultFont((CEGUI::utf8*)"BlueHighway-12");
		CEGUI::MouseCursor::getSingleton().setImage("TaharezLook", "MouseArrow");
		CEGUI::MouseCursor::getSingleton().show( );
		setupEventHandlers();

		Entity* ogreHead = mSceneMgr->createEntity("Ninja", "ninja.mesh");
		ogreHead->setCastShadows( true ); // ׸  ѱ
		SceneNode* headNode = mSceneMgr->getRootSceneNode()->createChildSceneNode();
		headNode->attachObject(ogreHead);

		//Plane ߰
		Plane plane(Vector3::UNIT_Y, 0);
		MeshManager::getSingleton().createPlane("ground", ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, plane,
			1500, 1500, 20, 20, true, 1, 5, 5, Vector3::UNIT_Z);
		Entity *ent = mSceneMgr->createEntity("GroundEntity", "ground");
		mSceneMgr->getRootSceneNode()->createChildSceneNode()->attachObject(ent);
		ent->setMaterialName("Examples/Rockwall");
		ent->setCastShadows( false );


		// Set ambient light
		mSceneMgr->setAmbientLight(ColourValue(0, 0, 0));
		mSceneMgr->setShadowTechnique( SHADOWTYPE_STENCIL_ADDITIVE ); //׸ ũ


		// Create a light
		Light* l = mSceneMgr->createLight("MainLight");
		l->setPosition(20,80,50);

		//Ʈ Ʈ ߰
		l = mSceneMgr->createLight("Light1");
		l->setType( Light::LT_POINT );
		l->setPosition( Vector3(0, 150, 250));
		l->setDiffuseColour(ColourValue(1.0, 0, 0));
		l->setSpecularColour(ColourValue(1.0, 0, 0));

		//𷺼ų Ʈ ߰
		l = mSceneMgr->createLight("Light2");
		l->setType( Light::LT_DIRECTIONAL);
		l->setDiffuseColour(ColourValue(0.25, 0.25, 0));
		l->setSpecularColour(ColourValue(0.25, 0.25, 0));
		l->setDirection(Vector3(0, -1, 1));

		//Ʈ Ʈ ߰

		l = mSceneMgr->createLight("Light3");
		l->setType( Light::LT_SPOTLIGHT);
		l->setDiffuseColour(ColourValue(0, 0, 1.0));
		l->setSpecularColour(ColourValue(0, 0, 1.0));
		l->setDirection(Vector3(-1, -1, 0));
		l->setPosition(Vector3(300, 300, 0));
		l->setSpotlightRange( Degree(30), Degree(50));

	}

	// Create new frame listener
	void createFrameListener(void)
	{
		mFrameListener= new tutorial2FrameListener(mSceneMgr, mWindow, mCamera, mGUIRenderer);
		mRoot->addFrameListener(mFrameListener);
	}
	void setupEventHandlers(void)
	{
	}

	bool handleQuit(const CEGUI::EventArgs& e)
	{
		static_cast<tutorial2FrameListener*>(mFrameListener)->requestShutdown();
		return true;
	}

	virtual void createViewports(void)
	{
		Viewport *vp = mWindow->addViewport(mCamera); //Ʈ 
		vp->setBackgroundColour(ColourValue(0.3, 0.1, 0.7)); //׶ ÷ 
		mCamera->setAspectRatio( Real(vp->getActualWidth()) / Real(vp->getActualHeight()) );
		
	}
};

#endif // #ifndef __tutorial2_h_